@@ -1,4 +1,4 @@
-/***************************************************************************
+/*************************************************************** -*- c++ -*-
  *       Copyright (c) 2003,2004 by Marcel Wiesweg                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -65,11 +65,12 @@ int Storage::cleanSubDir(const char *dir) {
    static bool reportedError=false; //avoid filling up syslog
    DIR *d=opendir(dir);
    bool hadError=false;
-   int bytesDeleted=0, filesize;
+   int bytesDeleted=0;
    if (d) {
       struct dirent *txtfile, path;
       struct stat txtfilestat;
       char fullPath[PATH_MAX];
+      int filesize;
       while ( (!readdir_r(d, &path, &txtfile) && txtfile != NULL) ) {
          int len=strlen(txtfile->d_name);
          //check that the file end with .vtx to avoid accidents and disasters
@@ -100,9 +101,9 @@ int Storage::cleanSubDir(const char *dir) {
    return bytesDeleted;
 }
 
-Storage::Storage() {
-   byteCount=0;
-   failedFreeSpace=false;
+Storage::Storage()
+  : byteCount(0), failedFreeSpace(false)
+{
 }
 
 Storage::~Storage() {
@@ -128,9 +129,9 @@ void Storage::freeSpace() {
    time_t min=time(0);
    char minDir[PATH_MAX];
    char fullPath[PATH_MAX];
-   int haveDir=0;
    DIR *top=opendir(getRootDir());
    if (top) {
+      int haveDir=0;
       struct dirent *chandir, path;
       struct stat chandirstat;
       while ( (!readdir_r(top, &path, &chandir) && chandir != NULL) ) {
@@ -179,9 +180,9 @@ void Storage::prepareDirectory(tChannelID chan) {
 
 #define TELETEXT_PAGESIZE 972
 
-LegacyStorage::LegacyStorage(int maxMB) {
-   fsBlockSize=1;
-   pageBytes=TELETEXT_PAGESIZE;
+LegacyStorage::LegacyStorage(int maxMB)
+  : fsBlockSize(1), pageBytes(TELETEXT_PAGESIZE)
+{
    initMaxStorage(maxMB);
 }
 
@@ -473,9 +474,8 @@ bool cTelePage::IsTopTextPage()
 }
 
 cTxtStatus::cTxtStatus(bool storeTopText, Storage* storage)
- :storeTopText(storeTopText), storage(storage)
+   : receiver(NULL), storeTopText(storeTopText), storage(storage)
 {
-   receiver = NULL;
 }
 
 cTxtStatus::~cTxtStatus()
@@ -483,8 +483,22 @@ cTxtStatus::~cTxtStatus()
     delete receiver;
 }
 
-void cTxtStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber)
+void cTxtStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber
+#if APIVERSNUM >= 10726
+                               , bool LiveView
+#endif
+                               )
 {
+#if APIVERSNUM < 10726
+   bool LiveView = Device->IsPrimaryDevice();
+#endif
+#if APIVERSNUM >= 10725
+   // Disconnect receiver if channel is 0, will reconnect to new
+   // receiver after channel change.
+   if (LiveView && ChannelNumber == 0)
+      DELETENULL(receiver);
+#endif
+
    // ignore if channel is 0
    if (ChannelNumber == 0) return;
 
@@ -494,19 +508,17 @@ void cTxtStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber)
    if (newLiveChannel == NULL) return;
 
    // ignore non-live-channel-switching
-   if (!Device->IsPrimaryDevice() ||
-      ChannelNumber != cDevice::CurrentChannel()) return;
+   if (!LiveView || ChannelNumber != cDevice::CurrentChannel()) return;
 
    // live channel was changed
    // now re-attach the receiver to the new live channel
 
-   delete receiver;
-   receiver = NULL;
+   DELETENULL(receiver);
 
    int TPid = newLiveChannel->Tpid();
 
    if (TPid) {
-      receiver = new cTxtReceiver(TPid, newLiveChannel->GetChannelID(), storeTopText, storage);
+      receiver = new cTxtReceiver(newLiveChannel, storeTopText, storage);
       cDevice::ActualDevice()->AttachReceiver(receiver);
    }
 
@@ -514,10 +526,18 @@ void cTxtStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber)
 }
 
 
-cTxtReceiver::cTxtReceiver(int TPid, tChannelID chan, bool storeTopText, Storage* storage)
- : cReceiver(chan, -1, TPid), cThread("osdteletext-receiver"),
+cTxtReceiver::cTxtReceiver(const cChannel* chan, bool storeTopText, Storage* storage)
+#if APIVERSNUM >= 10712
+ : cReceiver(chan, -1), cThread("osdteletext-receiver"),
+#else
+ : cReceiver(chan, -1, chan->Tpid()), cThread("osdteletext-receiver"),
+#endif
    TxtPage(0), storeTopText(storeTopText), buffer((188+60)*75), storage(storage)
 {
+#if APIVERSNUM >= 10712
+   SetPids(NULL);
+   AddPid(chan->Tpid());
+#endif
    storage->prepareDirectory(ChannelID());
 
    // 10 ms timeout on getting TS frames
@@ -597,8 +617,7 @@ void cTxtReceiver::SaveAndDeleteTxtPage()
   if (TxtPage) {
      if (storeTopText || !TxtPage->IsTopTextPage()) {
         TxtPage->save();
-        delete TxtPage;
-        TxtPage=NULL;
+        DELETENULL(TxtPage);
      }
   }
 }
